@@ -12,9 +12,10 @@
 // extern "C" {
 // #endif
 
-#include <R.h>
+/*#include <R.h>
 #include <Rdefines.h>
-#include <Rinternals.h>
+#include <Rinternals.h>*/
+#include "rgdal.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -113,7 +114,7 @@ getGDALRasterPtr(SEXP sxpRasterBand) {
 
 }
 
-static void
+/*static void
 __errorHandler(CPLErr eErrClass, int err_no, const char *msg) {
 
   if (eErrClass == CE_Warning) {
@@ -128,14 +129,14 @@ __errorHandler(CPLErr eErrClass, int err_no, const char *msg) {
 
   return;
 
-}
+}*/
 
 /************************************************************************/
 /*                       CPLDefaultErrorHandler()                       */
 /* quoted from GDAL, pointed at REprintf()                              */
 /************************************************************************/
 
-void CPL_STDCALL R_CPLDefaultErrorHandler( CPLErr eErrClass, int nError, 
+/*void CPL_STDCALL R_CPLDefaultErrorHandler( CPLErr eErrClass, int nError, 
                              const char * pszErrorMsg )
 
 {
@@ -172,17 +173,63 @@ void CPL_STDCALL R_CPLDefaultErrorHandler( CPLErr eErrClass, int nError,
                  nMaxErrors );
     }
 
+}*/
+
+static void
+__errorHandler(CPLErr eErrClass, int err_no, const char *msg) {
+        saved_eErrClass = eErrClass;
+        saved_err_no = err_no;
+/* a mutex could be usefull here to avoid a race condition if 2 threads 
+trigger the error handler at the same time */
+        strncpy(saved_error_msg, msg, sizeof(saved_error_msg));
+        saved_error_msg[sizeof(saved_error_msg)-1] = 0;
 }
 
+void installErrorHandler()
+{
+   CPLPushErrorHandler(__errorHandler);
+   saved_err_no = 0;
+}
+
+void uninstallErrorHandlerAndTriggerError()
+{
+    CPLPopErrorHandler();
+    if (saved_err_no == CE_Warning) {
+
+    warning("\n\tNon-fatal GDAL Error %d: %s\n", saved_err_no, 
+saved_error_msg);
+
+  } else if (saved_err_no == CE_Failure) {
+
+    error("\n\tGDAL Error %d: %s\n", saved_err_no, saved_error_msg);
+
+  }
+}
+
+
 
 SEXP
 RGDAL_Init(void) {
 
-  CPLSetErrorHandler((CPLErrorHandler)__errorHandler);
+//  CPLSetErrorHandler((CPLErrorHandler)__errorHandler);
+//  CPLPushErrorHandler((CPLErrorHandler)__errorHandler);
 
+    installErrorHandler();
   GDALAllRegister();
+    uninstallErrorHandlerAndTriggerError();
 
+    installErrorHandler();
   OGRRegisterAll();
+    uninstallErrorHandlerAndTriggerError();
+ 
+  return(R_NilValue);
+
+}
+
+SEXP
+RGDAL_Exit(void) {
+
+//  CPLPopErrorHandler();
  
   return(R_NilValue);
 
@@ -193,7 +240,10 @@ RGDAL_GDALVersionInfo(SEXP str) {
     SEXP ans;
 
     PROTECT(ans=NEW_CHARACTER(1));
+
+    installErrorHandler();
     SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(GDALVersionInfo(asString(str))));
+    uninstallErrorHandlerAndTriggerError();
 
     UNPROTECT(1);
 
@@ -205,7 +255,9 @@ RGDAL_GDAL_DATA_Info(void) {
     SEXP ans;
 
     PROTECT(ans=NEW_CHARACTER(1));
+    installErrorHandler();
     SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(CSVFilename( "prime_meridian.csv" )));
+    uninstallErrorHandlerAndTriggerError();
 
     UNPROTECT(1);
 
@@ -227,7 +279,9 @@ RGDAL_GetDescription(SEXP sxpObj) {
 
   void *pGDALObj = getGDALObjPtr(sxpObj);
 
+  installErrorHandler();
   const char *desc = ((GDALMajorObject *)pGDALObj)->GetDescription();
+  uninstallErrorHandlerAndTriggerError();
 
   return(mkString_safe(desc));
 
@@ -239,7 +293,9 @@ RGDAL_GetDriverNames(void) {
 
   SEXP ans, ansnames;
   int pc=0;
+  installErrorHandler();
   int nDr=GDALGetDriverCount();
+  uninstallErrorHandlerAndTriggerError();
 
   PROTECT(ans = NEW_LIST(4)); pc++;
   PROTECT(ansnames = NEW_CHARACTER(4)); pc++;
@@ -255,6 +311,7 @@ RGDAL_GetDriverNames(void) {
   SET_VECTOR_ELT(ans, 3, NEW_LOGICAL(nDr));
 
   int i, flag;
+  installErrorHandler();
   for (i = 0; i < nDr; ++i) {
 
     GDALDriver *pDriver = GetGDALDriverManager()->GetDriver(i);
@@ -270,6 +327,7 @@ RGDAL_GetDriverNames(void) {
     if (GDALGetMetadataItem( pDriver, GDAL_DCAP_CREATECOPY, NULL )) flag=1;
     LOGICAL_POINTER(VECTOR_ELT(ans, 3))[i] = flag;
   }
+  uninstallErrorHandlerAndTriggerError();
 
   UNPROTECT(pc);
 
@@ -282,7 +340,9 @@ RGDAL_GetDriver(SEXP sxpDriverName) {
 
   const char *pDriverName = asString(sxpDriverName);
 
+  installErrorHandler();
   GDALDriver *pDriver = (GDALDriver *) GDALGetDriverByName(pDriverName);
+  uninstallErrorHandlerAndTriggerError();
 
   if (pDriver == NULL)
     error("No driver registered with name: %s\n", pDriverName);
@@ -304,12 +364,14 @@ deleteFile(GDALDriver *pDriver, const char *filename) {
 //  fflush(stderr);
 #endif
 
+  installErrorHandler();
   if (strcmp(GDALGetDriverLongName( pDriver ), "In Memory Raster") != 0) {
       CPLErr eErr = pDriver->Delete(filename);
 
     if (eErr == CE_Failure)
       warning("Failed to delete dataset: %s\n", filename);
   }
+  uninstallErrorHandlerAndTriggerError();
 
 #ifdef RGDALDEBUG
   Rprintf("done.\n", filename);
@@ -343,6 +405,7 @@ RGDAL_CloseHandle(SEXP sxpHandle) {
   Rprintf("Closing GDAL dataset handle %p... ", (void *) pDataset);
 #endif
 
+  installErrorHandler();
   if (pDataset != NULL) {
 
     pDataset->~GDALDataset();
@@ -354,6 +417,7 @@ RGDAL_CloseHandle(SEXP sxpHandle) {
 #endif
 
   }
+  uninstallErrorHandlerAndTriggerError();
 
 #ifdef RGDALDEBUG
   Rprintf("done.\n");
@@ -371,6 +435,7 @@ RGDAL_DeleteHandle(SEXP sxpHandle) {
 
   if (pDataset == NULL) return(R_NilValue);
 
+  installErrorHandler();
   GDALDriver *pDriver = pDataset->GetDriver();
 
   const char *filename = pDataset->GetDescription();
@@ -381,6 +446,7 @@ RGDAL_DeleteHandle(SEXP sxpHandle) {
 #ifndef OSGEO4W
   deleteFile(pDriver, filename);
 #endif
+  uninstallErrorHandlerAndTriggerError();
   return(R_NilValue);
 
 }
@@ -427,22 +493,30 @@ RGDAL_CreateDataset(SEXP sxpDriver, SEXP sDim, SEXP sType,
 
   GDALDataType eGDALType = (GDALDataType) asInteger(sType);
 
+  installErrorHandler();
   for (i=0; i < length(sOpts); i++) papszCreateOptions = CSLAddString( 
     papszCreateOptions, CHAR(STRING_ELT(sOpts, i)) );
 #ifdef RGDALDEBUG
   for (i=0; i < CSLCount(papszCreateOptions); i++)
     Rprintf("option %d: %s\n", i, CSLGetField(papszCreateOptions, i));
 #endif
+  uninstallErrorHandlerAndTriggerError();
+  installErrorHandler();
   pDataset = pDriver->Create(filename,
 			  INTEGER(sDim)[0],
 			  INTEGER(sDim)[1],
 			  INTEGER(sDim)[2],
 			  eGDALType, papszCreateOptions);
+  uninstallErrorHandlerAndTriggerError();
+  installErrorHandler();
   CSLDestroy(papszCreateOptions);
+  uninstallErrorHandlerAndTriggerError();
 
   if (pDataset == NULL) error("Unable to create dataset\n");
 
+  installErrorHandler();
   pDataset->SetDescription(filename);
+  uninstallErrorHandlerAndTriggerError();
 
   SEXP sxpHandle = R_MakeExternalPtr((void *) pDataset,
 				     mkChar("GDAL Dataset"),
@@ -470,27 +544,31 @@ RGDAL_OpenDataset(SEXP filename, SEXP read_only, SEXP silent) {
   if (asLogical(silent))
     CPLPushErrorHandler(CPLQuietErrorHandler);
   else
-    CPLPushErrorHandler(R_CPLDefaultErrorHandler); 
+     installErrorHandler();
 
   GDALDataset *pDataset = (GDALDataset *) GDALOpen(fn, RWFlag);
 
-  CPLPopErrorHandler();
+  if (pDataset == NULL)
+    error("%s\n", CPLGetLastErrorMsg());
+
+  if (asLogical(silent))
+    CPLPopErrorHandler();
+  else
+    uninstallErrorHandlerAndTriggerError();
 
 /* Similarly to SWIG bindings, the following lines will cause
 RGDAL_OpenDataset() to fail on - uncleared - errors even if pDataset is not
 NULL. They could also be just removed. While pDataset != NULL, there's some
 hope ;-) */
 
-  CPLErr eclass = CPLGetLastErrorType();
+/*  CPLErr eclass = CPLGetLastErrorType();
 
   if (pDataset != NULL && eclass == CE_Failure) {
     GDALClose(pDataset);
     pDataset = NULL;
     __errorHandler(eclass, CPLGetLastErrorNo(), CPLGetLastErrorMsg());
-  }
+  }*/
 
-  if (pDataset == NULL)
-    error("%s\n", CPLGetLastErrorMsg());
 
   SEXP sxpHandle = R_MakeExternalPtr((void *) pDataset,
 				     mkChar("GDAL Dataset"),
@@ -515,18 +593,27 @@ RGDAL_CopyDataset(SEXP sxpDataset, SEXP sxpDriver,
 
   GDALDriver *pDriver = getGDALDriverPtr(sxpDriver);
 
+  installErrorHandler();
   for (i=0; i < length(sxpOpts); i++) papszCreateOptions = CSLAddString( 
     papszCreateOptions, CHAR(STRING_ELT(sxpOpts, i)) );
+  uninstallErrorHandlerAndTriggerError();
 #ifdef RGDALDEBUG
+  installErrorHandler();
   for (i=0; i < CSLCount(papszCreateOptions); i++)
     Rprintf("option %d: %s\n", i, CSLGetField(papszCreateOptions, i));
+  uninstallErrorHandlerAndTriggerError();
 #endif
+  installErrorHandler();
   GDALDataset *pDatasetCopy = pDriver->CreateCopy(filename,
 		pDataset, asInteger(sxpStrict),
 		papszCreateOptions, NULL, NULL);
+  uninstallErrorHandlerAndTriggerError();
 
   if (pDatasetCopy == NULL) error("Dataset copy failed\n");
+
+  installErrorHandler();
   CSLDestroy(papszCreateOptions);
+  uninstallErrorHandlerAndTriggerError();
 
   SEXP sxpHandle = R_MakeExternalPtr((void *) pDatasetCopy,
 				     mkChar("GDAL Dataset"),
@@ -542,7 +629,12 @@ RGDAL_GetRasterXSize(SEXP sDataset) {
 
   GDALDataset *pDataset = getGDALDatasetPtr(sDataset);
 
-  return(ScalarInteger(pDataset->GetRasterXSize()));
+  int res;
+
+  installErrorHandler();
+  res = pDataset->GetRasterXSize();
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarInteger(res));
 
 }
 
@@ -551,7 +643,12 @@ RGDAL_GetRasterYSize(SEXP sDataset) {
 
   GDALDataset *pDataset = getGDALDatasetPtr(sDataset);
 
-  return(ScalarInteger(pDataset->GetRasterYSize()));
+  int res;
+
+  installErrorHandler();
+  res = pDataset->GetRasterYSize();
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarInteger(res));
 
 }
 
@@ -560,7 +657,12 @@ RGDAL_GetRasterCount(SEXP sDataset) {
 
   GDALDataset *pDataset = getGDALDatasetPtr(sDataset);
 
-  return(ScalarInteger(pDataset->GetRasterCount()));
+  int res;
+
+  installErrorHandler();
+  res = pDataset->GetRasterCount();
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarInteger(res));
 
 }
 
@@ -574,14 +676,20 @@ RGDAL_GetProjectionRef(SEXP sDataset) {
 
   GDALDataset *pDataset = getGDALDatasetPtr(sDataset);
   
+  installErrorHandler();
   pszSRS_WKT = (char*) pDataset->GetProjectionRef();
+  uninstallErrorHandlerAndTriggerError();
 
+  installErrorHandler();
   oSRS.importFromWkt( &pszSRS_WKT );
   oSRS.exportToProj4( &pszSRS_WKT );
+  uninstallErrorHandlerAndTriggerError();
   PROTECT(ans = NEW_CHARACTER(1));
   SET_STRING_ELT(ans, 0, COPY_TO_USER_STRING(pszSRS_WKT));
 
+  installErrorHandler();
   CPLFree( pszSRS_WKT );
+  uninstallErrorHandlerAndTriggerError();
   UNPROTECT(1);
   return(ans);
 
@@ -596,11 +704,13 @@ RGDAL_GetMetadata(SEXP sDataset, SEXP tag) {
 
     GDALDataset *pDataset = getGDALDatasetPtr(sDataset);
 
+  installErrorHandler();
     if (tag == R_NilValue) {
         papszMetadata = pDataset->GetMetadata( NULL );
     } else {
         papszMetadata = pDataset->GetMetadata(CHAR(STRING_ELT(tag, 0)));
     }
+  uninstallErrorHandlerAndTriggerError();
 
     if (CSLCount(papszMetadata) == 0) return(R_NilValue);
 
@@ -622,7 +732,9 @@ RGDAL_GetDatasetDriver(SEXP sDataset) {
 
   GDALDataset *pDataset = getGDALDatasetPtr(sDataset);
 
+  installErrorHandler();
   GDALDriver *pDriver = pDataset->GetDriver();
+  uninstallErrorHandlerAndTriggerError();
 
   SEXP sxpDriver = R_MakeExternalPtr((void *) pDriver,
 				     mkChar("GDAL Dataset"),
@@ -637,7 +749,10 @@ RGDAL_GetDriverShortName(SEXP sxpDriver) {
 
   GDALDriver *pDriver = getGDALDriverPtr(sxpDriver);
 
-  return(mkString_safe(GDALGetDriverShortName( pDriver )));
+  installErrorHandler();
+  const char *desc = GDALGetDriverShortName( pDriver );
+  uninstallErrorHandlerAndTriggerError();
+  return(mkString_safe(desc));
 
 }
 
@@ -645,8 +760,10 @@ SEXP
 RGDAL_GetDriverLongName(SEXP sxpDriver) {
 
   GDALDriver *pDriver = getGDALDriverPtr(sxpDriver);
-
-  return(mkString_safe(GDALGetDriverLongName( pDriver )));
+  installErrorHandler();
+  const char *desc = GDALGetDriverLongName( pDriver );
+  uninstallErrorHandlerAndTriggerError();
+  return(mkString_safe(desc));
 }
 
 SEXP
@@ -656,7 +773,9 @@ RGDAL_GetRasterBand(SEXP sDataset, SEXP sBand) {
 
   int band = asInteger(sBand);
 
+  installErrorHandler();
   GDALRasterBand *pRasterBand = pDataset->GetRasterBand(band);
+  uninstallErrorHandlerAndTriggerError();
 
   SEXP rpRasterBand = R_MakeExternalPtr((void *) pRasterBand,
 					mkChar("GDAL Raster Band"),
@@ -670,7 +789,12 @@ RGDAL_GetXSize(SEXP sRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sRasterBand);
 
-  return(ScalarInteger(pRasterBand->GetXSize()));
+  int res;
+
+  installErrorHandler();
+  res = pRasterBand->GetXSize();
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarInteger(res));
 
 }
 
@@ -679,7 +803,12 @@ RGDAL_GetYSize(SEXP sRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sRasterBand);
 
-  return(ScalarInteger(pRasterBand->GetYSize()));
+  int res;
+
+  installErrorHandler();
+  res = pRasterBand->GetYSize();
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarInteger(res));
 
 }
 
@@ -690,7 +819,9 @@ RGDAL_GetRasterBlockSize(SEXP rasterObj) {
 	 
 	 SEXP blockSize = allocVector(INTSXP, 2);
 	 
+  installErrorHandler();
 	 raster->GetBlockSize(INTEGER(blockSize) + 1, INTEGER(blockSize));
+  uninstallErrorHandlerAndTriggerError();
 	 
 	 return(blockSize);
 	 
@@ -700,8 +831,12 @@ SEXP
 RGDAL_GetAccess(SEXP sxpDataset) {
 
   GDALDataset *pDataset = getGDALDatasetPtr(sxpDataset);
+  int res;
 
-  return(ScalarLogical(pDataset->GetAccess() == GA_ReadOnly));
+  installErrorHandler();
+  res = pDataset->GetAccess() == GA_ReadOnly;
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarLogical(res));
 
 }
 
@@ -710,7 +845,12 @@ RGDAL_GetRasterAccess(SEXP sxpRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
-  return(ScalarLogical(pRasterBand->GetAccess() == GA_ReadOnly));
+  int res;
+
+  installErrorHandler();
+  res = pRasterBand->GetAccess() == GA_ReadOnly;
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarLogical(res));
 
 }
 
@@ -721,7 +861,9 @@ RGDAL_GetNoDataValue(SEXP sxpRasterBand) {
 
   int hasNoDataValue;
 
+  installErrorHandler();
   double noDataValue = pRasterBand->GetNoDataValue(&hasNoDataValue);
+  uninstallErrorHandlerAndTriggerError();
 
   return(hasNoDataValue ? ScalarReal(noDataValue) : R_NilValue);
 
@@ -731,8 +873,12 @@ SEXP
 RGDAL_GetOffset(SEXP sxpRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
+  double res;
 
-  return(ScalarReal(pRasterBand->GetOffset()));
+  installErrorHandler();
+  res = pRasterBand->GetOffset();
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarReal(res));
 
 }
 
@@ -740,8 +886,12 @@ SEXP
 RGDAL_GetScale(SEXP sxpRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
+  double res;
 
-  return(ScalarReal(pRasterBand->GetScale()));
+  installErrorHandler();
+  res = pRasterBand->GetScale();
+  uninstallErrorHandlerAndTriggerError();
+  return(ScalarReal(res));
 
 }
 
@@ -797,19 +947,26 @@ RGDAL_GetRAT(SEXP sxpRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
+  installErrorHandler();
   const GDALRasterAttributeTable *pRAT = pRasterBand->GetDefaultRAT();
+  uninstallErrorHandlerAndTriggerError();
 
   if (pRAT == NULL) return(R_NilValue);
 
+  installErrorHandler();
   nc = (int) pRAT->GetColumnCount();
+  uninstallErrorHandlerAndTriggerError();
   PROTECT(ans = NEW_LIST(nc));np++;
   PROTECT(nc_names = NEW_CHARACTER(nc));np++;
   nc_types = (GDALRATFieldType *) R_alloc((size_t) nc,
     sizeof(GDALRATFieldType));
   nc_usages = (GDALRATFieldUsage *) R_alloc((size_t) nc,
     sizeof(GDALRATFieldUsage));
+  installErrorHandler();
   nr = (int) pRAT->GetRowCount();
+  uninstallErrorHandlerAndTriggerError();
 
+  installErrorHandler();
   for (i=0; i<nc; i++) {
     nc_types[i] = pRAT->GetTypeOfCol(i);
     nc_usages[i] = pRAT->GetUsageOfCol(i);
@@ -824,6 +981,8 @@ RGDAL_GetRAT(SEXP sxpRasterBand) {
       error("unknown column type");
     }
   }
+  uninstallErrorHandlerAndTriggerError();
+  installErrorHandler();
   for (i=0; i<nc; i++) {
 
     if (nc_types[i] == GFT_Integer) {
@@ -850,6 +1009,7 @@ RGDAL_GetRAT(SEXP sxpRasterBand) {
     }     
 
   }
+  uninstallErrorHandlerAndTriggerError();
   PROTECT(GFT_type = NEW_CHARACTER(nc));np++;
   PROTECT(GFT_usage = NEW_CHARACTER(nc));np++;
 
@@ -876,7 +1036,9 @@ RGDAL_GetBandMinimum(SEXP sxpRasterBand) {
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
   PROTECT(ans = NEW_NUMERIC(1));
 
+  installErrorHandler();
   NUMERIC_POINTER(ans)[0] = (double) pRasterBand->GetMinimum();
+  uninstallErrorHandlerAndTriggerError();
 
   UNPROTECT(1);
   return(ans);
@@ -890,7 +1052,9 @@ RGDAL_GetBandMaximum(SEXP sxpRasterBand) {
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
   PROTECT(ans = NEW_NUMERIC(1));
 
+  installErrorHandler();
   NUMERIC_POINTER(ans)[0] = (double) pRasterBand->GetMaximum();
+  uninstallErrorHandlerAndTriggerError();
 
   UNPROTECT(1);
   return(ans);
@@ -907,19 +1071,23 @@ RGDAL_GetBandStatistics(SEXP sxpRasterBand, SEXP silent) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
+  installErrorHandler();
   err = pRasterBand->GetStatistics(FALSE, FALSE, &min, &max, &mean, &sd);
 
   if (err == CE_Failure) {
 	if (!LOGICAL_POINTER(silent)[0])
             warning("statistics not supported by this driver");
+  uninstallErrorHandlerAndTriggerError();
         return(R_NilValue);
   }
 
   if (err == CE_Warning) {
 	if (!LOGICAL_POINTER(silent)[0])
     	    warning("statistics not supported by this driver");
+  uninstallErrorHandlerAndTriggerError();
         return(R_NilValue);
   }
+  uninstallErrorHandlerAndTriggerError();
 
   PROTECT(ans = NEW_NUMERIC(4));
   NUMERIC_POINTER(ans)[0] = min;
@@ -941,7 +1109,9 @@ RGDAL_GetBandType(SEXP sxpRasterBand) {
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
   PROTECT(ans = NEW_INTEGER(1));
 
+  installErrorHandler();
   INTEGER_POINTER(ans)[0] = (int) pRasterBand->GetRasterDataType();
+  uninstallErrorHandlerAndTriggerError();
 
   UNPROTECT(1);
   return(ans);
@@ -970,6 +1140,7 @@ RGDAL_PutRasterData(SEXP sxpRasterBand, SEXP sxpData, SEXP sxpOffset) {
     PROTECT(sxpData = coerceVector(sxpData, INTSXP));
   // Transpose data
 // replication for 2.4.0 RSB 20060726
+    installErrorHandler();
     if(pRasterBand->RasterIO(GF_Write,
 			   INTEGER(sxpOffset)[1],
 			   INTEGER(sxpOffset)[0],
@@ -978,8 +1149,11 @@ RGDAL_PutRasterData(SEXP sxpRasterBand, SEXP sxpData, SEXP sxpOffset) {
 			   rowsIn, colsIn,
 			   eGDALType,
 			   0, 0)
-       == CE_Failure)
+       == CE_Failure) {
+      uninstallErrorHandlerAndTriggerError();
       error("Failure during raster IO\n");
+    }
+    uninstallErrorHandlerAndTriggerError();
 
     break;
 
@@ -989,6 +1163,7 @@ RGDAL_PutRasterData(SEXP sxpRasterBand, SEXP sxpData, SEXP sxpOffset) {
     eGDALType = GDAL_FLOAT_TYPE;
     PROTECT(sxpData = coerceVector(sxpData, REALSXP));
   // Transpose data
+    installErrorHandler();
     if(pRasterBand->RasterIO(GF_Write,
 			   INTEGER(sxpOffset)[1],
 			   INTEGER(sxpOffset)[0],
@@ -997,8 +1172,11 @@ RGDAL_PutRasterData(SEXP sxpRasterBand, SEXP sxpData, SEXP sxpOffset) {
 			   rowsIn, colsIn,
 			   eGDALType,
 			   0, 0)
-       == CE_Failure)
+       == CE_Failure) {
+      uninstallErrorHandlerAndTriggerError();
       error("Failure during raster IO\n");
+    }
+    uninstallErrorHandlerAndTriggerError();
 
     break;
 
@@ -1010,6 +1188,7 @@ RGDAL_PutRasterData(SEXP sxpRasterBand, SEXP sxpData, SEXP sxpOffset) {
     eGDALType = GDAL_COMPLEX_TYPE;
     PROTECT(sxpData = coerceVector(sxpData, CPLXSXP));
   // Transpose data
+    installErrorHandler();
     if(pRasterBand->RasterIO(GF_Write,
 			   INTEGER(sxpOffset)[1],
 			   INTEGER(sxpOffset)[0],
@@ -1018,8 +1197,11 @@ RGDAL_PutRasterData(SEXP sxpRasterBand, SEXP sxpData, SEXP sxpOffset) {
 			   rowsIn, colsIn,
 			   eGDALType,
 			   0, 0)
-       == CE_Failure)
+       == CE_Failure) {
+      uninstallErrorHandlerAndTriggerError();
       error("Failure during raster IO\n");
+    }
+    uninstallErrorHandlerAndTriggerError();
 
     break;
     
@@ -1043,7 +1225,9 @@ RGDAL_GetBandNoDataValue(SEXP sxpRasterBand) {
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
   SEXP res;
   int hasNoDataValue;
+  installErrorHandler();
   double noDataValue = pRasterBand->GetNoDataValue(&hasNoDataValue);
+  uninstallErrorHandlerAndTriggerError();
 
   if (hasNoDataValue) {
     PROTECT(res = NEW_NUMERIC(1));
@@ -1068,7 +1252,9 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
   GDALDataType eGDALType = GDT_Int32;
   SEXPTYPE uRType = INTSXP;
 
+  installErrorHandler();
   int RDT = pRasterBand->GetRasterDataType();
+  uninstallErrorHandlerAndTriggerError();
 
   switch(RDT) {
 
@@ -1120,6 +1306,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
   switch(uRType) {
 
     case INTSXP:
+      installErrorHandler();
       if(pRasterBand->RasterIO(GF_Read,
 			   INTEGER(sxpRegion)[1],
 			   INTEGER(sxpRegion)[0],
@@ -1131,12 +1318,16 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
 			   eGDALType,
 			   INTEGER(sxpInterleave)[0],
 			   INTEGER(sxpInterleave)[1])
-         == CE_Failure)
+         == CE_Failure) {
+           uninstallErrorHandlerAndTriggerError();
            error("Failure during raster IO\n");
+      }
+      uninstallErrorHandlerAndTriggerError();
       break;
 
     case REALSXP:
 
+      installErrorHandler();
       if(pRasterBand->RasterIO(GF_Read,
 			   INTEGER(sxpRegion)[1],
 			   INTEGER(sxpRegion)[0],
@@ -1148,12 +1339,16 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
 			   eGDALType,
 			   INTEGER(sxpInterleave)[0],
 			   INTEGER(sxpInterleave)[1])
-         == CE_Failure)
+         == CE_Failure) {
+           uninstallErrorHandlerAndTriggerError();
            error("Failure during raster IO\n");
+      }
+      uninstallErrorHandlerAndTriggerError();
       break;
 
     case CPLXSXP:
 
+      installErrorHandler();
       if(pRasterBand->RasterIO(GF_Read,
 			   INTEGER(sxpRegion)[1],
 			   INTEGER(sxpRegion)[0],
@@ -1165,8 +1360,11 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
 			   eGDALType,
 			   INTEGER(sxpInterleave)[0],
 			   INTEGER(sxpInterleave)[1])
-         == CE_Failure)
+         == CE_Failure) {
+           uninstallErrorHandlerAndTriggerError();
            error("Failure during raster IO\n");
+      }
+      uninstallErrorHandlerAndTriggerError();
       break;
 
     default:
@@ -1244,6 +1442,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
     }
 
   } else {
+    installErrorHandler();
     if (uRType == REALSXP && pRasterBand->GetRasterDataType() == GDT_Float32) {
         for (i = 0; i < LENGTH(sRStorage); ++i)
 	  if (ISNAN(REAL(sRStorage)[i])) {
@@ -1251,6 +1450,7 @@ RGDAL_GetRasterData(SEXP sxpRasterBand,
 	  }
       
     }
+    uninstallErrorHandlerAndTriggerError();
   }
 
   UNPROTECT(pc);
@@ -1263,10 +1463,15 @@ RGDAL_GetPaletteInterp(SEXP sxpRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
+  installErrorHandler();
   GDALPaletteInterp ePI = 
     pRasterBand->GetColorTable()->GetPaletteInterpretation();
-
-  return(mkString_safe(GDALGetPaletteInterpretationName(ePI)));
+  uninstallErrorHandlerAndTriggerError();
+  
+  installErrorHandler();
+  const char *desc = GDALGetPaletteInterpretationName(ePI);
+  uninstallErrorHandlerAndTriggerError();
+  return(mkString_safe(desc));
 
 }
 
@@ -1275,19 +1480,27 @@ RGDAL_GetColorInterp(SEXP sxpRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
+  installErrorHandler();
   GDALColorInterp eCI = pRasterBand->GetColorInterpretation();
+  uninstallErrorHandlerAndTriggerError();
 
-  return(mkString_safe(GDALGetColorInterpretationName(eCI)));
+  installErrorHandler();
+  const char *desc = GDALGetColorInterpretationName(eCI);
+  uninstallErrorHandlerAndTriggerError();
+  return(mkString_safe(desc));
 
 }
 
 static SEXP
 GDALColorTable2Matrix(GDALColorTableH ctab) {
 
+        installErrorHandler();
 	int ncol = GDALGetColorEntryCount(ctab);
+        uninstallErrorHandlerAndTriggerError();
 
 	SEXP cmat = allocMatrix(INTSXP, ncol, 4);
 
+        installErrorHandler();
 	for (int i = 0; i < ncol; ++i) {
 
     	const GDALColorEntry* ce = GDALGetColorEntry(ctab, i);
@@ -1298,6 +1511,7 @@ GDALColorTable2Matrix(GDALColorTableH ctab) {
     	INTEGER(cmat)[i + 3 * ncol] = static_cast<int>(ce->c4);
 
   	}
+        uninstallErrorHandlerAndTriggerError();
 
   	return(cmat);
 	
@@ -1308,7 +1522,9 @@ RGDAL_GetColorTable(SEXP rasterObj) {
 
 	GDALRasterBandH rasterBand = getGDALRasterPtr(rasterObj);
 
+        installErrorHandler();
 	GDALColorTableH ctab = GDALGetRasterColorTable(rasterBand);
+        uninstallErrorHandlerAndTriggerError();
 
 	if (ctab == NULL) return(R_NilValue);
 
@@ -1325,12 +1541,16 @@ RGDAL_SetCategoryNames(SEXP sxpRasterBand, SEXP sxpNames) {
   char **nameList = NULL;
 
   int i;
+  installErrorHandler();
   for (i = 0; i < length(sxpNames); ++i)
     nameList = CSLAddString(nameList, asString(sxpNames, i));
+  uninstallErrorHandlerAndTriggerError();
 
+  installErrorHandler();
   CPLErr err = pRasterBand->SetCategoryNames(nameList);
 
   if (err == CE_Failure) warning("Failed to set category names");
+  uninstallErrorHandlerAndTriggerError();
 
   return(sxpRasterBand);
 
@@ -1341,24 +1561,33 @@ RGDAL_GetCategoryNames(SEXP sxpRasterBand) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
+  installErrorHandler();
   char **pcCNames = pRasterBand->GetCategoryNames();
+  uninstallErrorHandlerAndTriggerError();
 
   if (pcCNames == NULL) return(R_NilValue);
 
+  installErrorHandler();
   pcCNames = CSLDuplicate(pcCNames);
+  uninstallErrorHandlerAndTriggerError();
 
   SEXP sxpCNames;
 
-  PROTECT(sxpCNames = allocVector(STRSXP, CSLCount(pcCNames)));
+  installErrorHandler();
+  int ii = CSLCount(pcCNames);
+  uninstallErrorHandlerAndTriggerError();
+  PROTECT(sxpCNames = allocVector(STRSXP, ii));
 
   int i;
-  for (i = 0; i < CSLCount(pcCNames); ++i) {
+  installErrorHandler();
+  for (i = 0; i < ii; ++i) {
 
     const char *field = CSLGetField(pcCNames, i);
 
-    SET_VECTOR_ELT(sxpCNames, i, mkChar(field));
+    SET_STRING_ELT(sxpCNames, i, mkChar(field));
 
   }
+  uninstallErrorHandlerAndTriggerError();
 
   UNPROTECT(1);
   
@@ -1375,6 +1604,7 @@ RGDAL_GetGeoTransform(SEXP sxpDataset) {
   SEXP ceFail = NEW_LOGICAL(1);
   LOGICAL_POINTER(ceFail)[0] = FALSE;
 
+  installErrorHandler();
   CPLErr err = pDataset->GetGeoTransform(REAL(sxpGeoTrans));
 
   if (err == CE_Failure) {
@@ -1390,6 +1620,8 @@ RGDAL_GetGeoTransform(SEXP sxpDataset) {
 
   }
   setAttrib(sxpGeoTrans, install("CE_Failure"), ceFail);
+  uninstallErrorHandlerAndTriggerError();
+
   return(sxpGeoTrans);
 
 }
@@ -1400,10 +1632,12 @@ RGDAL_SetNoDataValue(SEXP sxpRasterBand, SEXP NoDataValue) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
+  installErrorHandler();
   err = pRasterBand->SetNoDataValue(NUMERIC_POINTER(NoDataValue)[0]);
 
   if (err == CE_Failure)
 	warning("setting of missing value not supported by this driver");
+  uninstallErrorHandlerAndTriggerError();
 
   return(sxpRasterBand);
 
@@ -1415,12 +1649,14 @@ SEXP RGDAL_SetStatistics(SEXP sxpRasterBand, SEXP statistics) {
 
   GDALRasterBand *pRasterBand = getGDALRasterPtr(sxpRasterBand);
 
+  installErrorHandler();
   err = pRasterBand->SetStatistics(NUMERIC_POINTER(statistics)[0],
     NUMERIC_POINTER(statistics)[1], NUMERIC_POINTER(statistics)[2],
     NUMERIC_POINTER(statistics)[3]);
 
   if (err == CE_Failure)
 	warning("setting of statistics not supported by this driver");
+  uninstallErrorHandlerAndTriggerError();
 
   return(sxpRasterBand);
 
@@ -1434,10 +1670,12 @@ RGDAL_SetGeoTransform(SEXP sxpDataset, SEXP GeoTransform) {
   if (LENGTH(GeoTransform) != 6)
 	error("GeoTransform argument should have length 6");
 
+  installErrorHandler();
   CPLErr err = pDataset->SetGeoTransform(NUMERIC_POINTER(GeoTransform));
 
   if (err == CE_Failure) 
 	warning("Failed to set GeoTransform\n");
+  uninstallErrorHandlerAndTriggerError();
 
   return(sxpDataset);
 }
@@ -1450,14 +1688,18 @@ RGDAL_SetProject(SEXP sxpDataset, SEXP proj4string) {
 
   GDALDataset *pDataset = getGDALDatasetPtr(sxpDataset);
 
+  installErrorHandler();
   oSRS.importFromProj4(CHAR(STRING_ELT(proj4string, 0)));
   oSRS.exportToWkt( &pszSRS_WKT );
+  uninstallErrorHandlerAndTriggerError();
 
+  installErrorHandler();
   OGRErr err = pDataset->SetProjection(pszSRS_WKT);
   CPLFree( pszSRS_WKT );
 
   if (err == CE_Failure) 
 	warning("Failed to set projection\n");
+  uninstallErrorHandlerAndTriggerError();
 
   return(sxpDataset);
 }
@@ -1476,24 +1718,38 @@ RGDAL_GenCMap(SEXP input1, SEXP input2, SEXP input3, SEXP output, SEXP nColors,
 	if (ncol < 2 || ncol > 256)
 		error("Number of colors should range from 2 to 256");
 	
+        installErrorHandler();
 	int err = GDALComputeMedianCutPCT(band1, band2, band3, NULL,
 	                                  ncol, &ctab, NULL, NULL); 
 	
-	if (err == CE_Failure) error("Error generating color table");
-	
+	if (err == CE_Failure) {
+          uninstallErrorHandlerAndTriggerError();
+          error("Error generating color table");
+	}
+        uninstallErrorHandlerAndTriggerError();
 	if (output != R_NilValue) {
 		
 		GDALRasterBand* target = getGDALRasterPtr(output);
 	
+                installErrorHandler();
 		err = GDALDitherRGB2PCT(band1, band2, band3, target, &ctab, NULL, NULL);
 	
-		if (err == CE_Failure) error("Image dithering failed");
+		if (err == CE_Failure) {
+                  uninstallErrorHandlerAndTriggerError();
+                  error("Image dithering failed");
+                }
+                uninstallErrorHandlerAndTriggerError();
 	
 		if (asLogical(setCMap)) {
 			
+                        installErrorHandler();
 			err = GDALSetRasterColorTable(target, &ctab);
 	
-			if (err == CE_Failure) warning("Unable to set color table");
+			if (err == CE_Failure) {
+                          uninstallErrorHandlerAndTriggerError();
+                          warning("Unable to set color table");
+                        }
+                        uninstallErrorHandlerAndTriggerError();
 			
 		}
 		
